import redis
import base64
import md5
import config
import sys

class UrlShortener:
    def __init__(self):
        self.redis = redis.StrictRedis(host=config.REDIS_HOST,
                                       port=config.REDIS_PORT,
                                       db=config.REDIS_DB)
        
    def shortcode(self, url):
        """
        Our main shortening function. The rationale here is that
        we are relying on the fact that for similarly sized inputs
        such as URLs the potential for collision in the 32 last bits
        of the MD5 hash is rather unlikely.
        
        The following things happen, in order:
        
        * compute the md5 digest of the given source
        * extract the lower 4 bytes
        * base64 encode the result
        * remove trailing padding if it exists
        
        Of course, should a collision happen, we will evict the previous
        key.
        
        """
        return base64.b64encode(md5.new(url).digest()[-4:]).replace('=','')

    def shorten(self, url):
        """
        The shortening workflow is very minimal. We try to
        set the redis key to the url value. We catch any
        exception in the process to properly report failures
        in the client
        """

        code = self.shortcode(url)
        
        try:
            self.redis.set(config.REDIS_PREFIX + code, url)
            return {'success': True,
                    'url': url,
                    'code': code,
                    'shorturl': config.URL_PREFIX + code}
        except:
            return {'success': False}

    def lookup(self, code):
        """
        The same strategy is used for the lookup than for the
        shortening. Here a None reply will imply either an
        error or a wrong code.
        """
        try:
            return self.redis.get(config.REDIS_PREFIX + code)
        except:
            return None

    
    




import logging
from flask import Flask, json, request, redirect, render_template, make_response
from shorten import UrlShortener
from functools import wraps
from bernhard import Client
from riemann_wrapper import riemann_wrapper

app = Flask(__name__)
shrt = UrlShortener()
logger = logging.getLogger()

logger.info("starting up")
riemann_client = Client(host=config.RIEMANN_HOST, port=config.RIEMANN_PORT)
riemann_client.send({'metric': 1, 'service': 'url-shortener.startup', 'ttl': 3600})
wrap_riemann = riemann_wrapper(client=riemann_client, prefix='url-shortener.')

## template pushing routes
@app.route('/')
@wrap_riemann('home')
def index():
    return render_template('index.html')

@app.route('/404')
@wrap_riemann('missing', tags=['http_404'])
def missing():
    return render_template('missing.html')

@app.route('/400')
@wrap_riemann('invalid', tags=['http_400'])
def invalid():
    return render_template('invalid')

## short url lookup
@app.route('/<code>')
@wrap_riemann('lookup')
def lookup(code):
    url = shrt.lookup(code)
    if not url:
        return redirect('/404')
    else:
        return redirect(url)
    
## short url generation
##
## If JSON is fed, we shorten and reply in JSON as well
## If a Form is posted we reply in HTML
## Otherwise we redirect to a failure page
@app.route('/', methods=['POST'])
@wrap_riemann('creation')
def shorten_url():
    if request.json and 'url' in request.json:
        url = request.json['url']
        res = shrt.shorten(request.json['url'])
        logger.debug("shortened %s to %s" % (url, res))
        response = make_response(json.dumps(res))
        response.headers['Content-Type'] = 'application/json'
        return response

    elif request.form and 'url' in request.form:
        url = request.form['url']
        res = shrt.shorten(request.form['url'])
        logger.debug("shortened %s to %s" % (url, res))
        return render_template('result.html', result=res)

    else:
        logger.info("invalid shorten request")
        return redirect('/400')



import os
import logging
import logging.config
from logstash_formatter import LogstashFormatter

REDIS_PREFIX = os.getenv("REDIS_PREFIX", "short:")
REDIS_HOST = os.getenv("REDIS_HOST", "localhost")
REDIS_PORT = int(os.getenv("REDIS_PORT", "6379"))
REDIS_DB = int(os.getenv("REDIS_DB", "0"))
URL_PREFIX = os.getenv("URL_PREFIX", "http://exo.po/")
LISTEN_HOST = os.getenv("LISTEN_HOST", "127.0.0.1")
LISTEN_PORT = int(os.getenv("LISTEN_PORT", "8080"))
RIEMANN_HOST = os.getenv("RIEMANN_HOST", "127.0.0.1")
RIEMANN_PORT = int(os.getenv("RIEMANN_PORT", "5555"))


LOG_FILE_PATH = os.getenv('LOG_FILE_PATH')
LOG_LEVEL = os.getenv('LOG_LEVEL', 'DEBUG')

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'logstash': {
            '()': LogstashFormatter,
        },
        'simple': {
            'format': '%(asctime)s %(levelname)s:%(name)s: %(message)s',
        },
    },
    'handlers': {
        'console': {
            'level': LOG_LEVEL,
            'class': 'logging.StreamHandler',
            'formatter': 'simple',
        },
        'logstash': {
            'level': LOG_LEVEL,
            'class': 'logging.NullHandler',
        },
        'null': {
            'class': 'logging.NullHandler',
        },
    },
    'root': {
        'handlers': ['console', 'logstash'],
        'level': 'DEBUG',
        'propagate': False,
    },
    'loggers': {
        'werkzeug': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'url_shortener': {
            'handlers': ['console', 'logstash'],
            'level': 'DEBUG',
            'propagate': False,
        },
    },
}
if LOG_FILE_PATH:
    LOGGING['handlers']['logstash'] = {
        'level': 'DEBUG',
        'class': 'logging.handlers.WatchedFileHandler',
        'formatter': 'logstash',
        'filename': LOG_FILE_PATH,
    }
    LOGGING['handlers']['console'] = {
        'class': 'logging.NullHandler',
    }
logging.config.dictConfig(LOGGING)
